#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#define DEPTH	      32
#define SPEED        2
#define NUMSTARS     100
#define FOG_DENSITY  0.02
#define PS           0.3
#define X_RANGE      50
#define Y_RANGE	   50
#define Z_RANGE		100
#define A_RANGE		10

UINT timer;

struct STAR {
  int x, y, z;
  int dax, day, daz;
  int ax, ay, az;
  GLfloat color[4];
} stars[NUMSTARS];

GLfloat color_sets[6][4] = {{0.0, 0.0, 1.0},
                            {0.0, 1.0, 0.0},
                            {0.0, 1.0, 1.0},
                            {1.0, 0.0, 0.0},
                            {1.0, 0.0, 1.0},
                            {1.0, 1.0, 0.0}};

HINSTANCE MainInstance;
int numstars = NUMSTARS;

void setDCPixelFormat(HDC hdc)
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hdc, &pfd);
  SetPixelFormat(hdc, pixelFormat, &pfd);
  DescribePixelFormat(hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

void InitializeGL()
{
  GLfloat lightAmbient[] = {0.3, 0.3, 0.3, 1.0};
  GLfloat lightDiffuse[] = {0.9, 0.9, 0.9, 1.0};
  GLfloat fogColor[4] = {0.9, 0.9, 1.0, 1.0};

  glFrontFace(GL_CW);
  glCullFace(GL_BACK);
  glEnable(GL_CULL_FACE);

  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.9, 0.9, 1.0, 0.0);

  glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glFogf(GL_FOG_MODE, GL_EXP2);
  glFogf(GL_FOG_DENSITY, FOG_DENSITY);
  glFogf(GL_FOG_END, Z_RANGE);
  glFogfv(GL_FOG_COLOR, fogColor);
  glEnable(GL_FOG);
}

void DrawRay()
{
  glBegin(GL_TRIANGLES);
    glNormal3f(0.0, PS, 0);
    glVertex3f(0.0, 1.0, 0.0);
    glVertex3f(PS, 0.0, PS);
    glVertex3f(-PS, 0.0, PS);
  glEnd();

  glBegin(GL_TRIANGLES);
    glNormal3f(PS, 0.0, 0.0);
    glVertex3f(0.0, 1.0, 0.0);
    glVertex3f(PS, 0.0, -PS);
    glVertex3f(PS, 0.0, PS);
  glEnd();

  glBegin(GL_TRIANGLES);
    glNormal3f(0.0, -PS, 0.0);
    glVertex3f(0.0, 1.0, 0.0);
    glVertex3f(-PS, 0.0, -PS);
    glVertex3f(PS, 0.0, -PS);
  glEnd();

  glBegin(GL_TRIANGLES);
    glNormal3f(-PS, 0.0, 0.0);
    glVertex3f(0.0, 1.0, 0.0);
    glVertex3f(-PS, 0.0, PS);
    glVertex3f(-PS, 0.0, -PS);
  glEnd();
}

void DrawStar()
{
  DrawRay();
  glTranslatef(0.0, -PS * 2, 0.0);
  glRotatef(180, 1.0, 0.0, 0.0);
  DrawRay();
  glTranslatef(PS, -PS, 0.0);
  glRotatef(-90, 0.0, 0.0, 1.0);
  DrawRay();
  glTranslatef(0.0, -PS * 2, 0.0);
  glRotatef(180, 0.0, 0.0, 1.0);
  DrawRay();
  glTranslatef(0.0, -PS, PS);
  glRotatef(90, 1.0, 0.0, 0.0);
  DrawRay();
  glTranslatef(0.0, -PS * 2, 0.0);
  glRotatef(180, 1.0, 0.0, 0.0);
  DrawRay();
}

void InitStar(int i)
{
  int n = random(6);
  stars[i].x = random(X_RANGE) - (X_RANGE >> 1);
  stars[i].y = random(Y_RANGE) - (Y_RANGE >> 1);
  stars[i].z = -random(Z_RANGE + DEPTH);
  stars[i].dax = random(A_RANGE) - (A_RANGE >> 1);
  stars[i].day = random(A_RANGE) - (A_RANGE >> 1);
  stars[i].daz = random(A_RANGE) - (A_RANGE >> 1);
  stars[i].ax = random(360) - 180;
  stars[i].ay = random(360) - 180;
  stars[i].az = random(360) - 180;
  for (int j = 0; j < 3; j++)
    stars[i].color[j] = color_sets[n][j];
  stars[i].color[3] = 1.0;
}

void drawScene(HDC hDC)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0, 0.0, -16.0);

  for (int i = 0; i < numstars; i++)
  {
    glPushMatrix();
    glTranslatef((GLfloat)stars[i].x, PS + (GLfloat)stars[i].y, (GLfloat)stars[i].z);
    glRotatef(stars[i].ax, 1.0, 0.0, 0.0);
    glRotatef(stars[i].ay, 0.0, 1.0, 0.0);
    glRotatef(stars[i].az, 0.0, 0.0, 1.0);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, stars[i].color);
    DrawStar();
    glPopMatrix();
    stars[i].z += SPEED;
    stars[i].ax += stars[i].dax;
    stars[i].ay += stars[i].day;
    stars[i].az += stars[i].daz;
    if (stars[i].z > DEPTH)
    {
      InitStar(i);
      stars[i].z = -100;
    }
  }

  SwapBuffers(hDC);
  glFlush();
}

LONG WINAPI dlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  char s[20];
  int n, i;

  switch (msg)
  {
    case WM_ACTIVATE:
      itoa(numstars, s, 10);
      SetDlgItemText(hWnd, 101, s);
      SetFocus(GetDlgItem(hWnd, 101)); return 0;
    case WM_CLOSE: EndDialog(hWnd, msg);
    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:
          GetDlgItemText(hWnd, 101, s, 19);
          n = atoi(s);
          if (n < 10 || n > 100)
          {
            MessageBox(hWnd, "      10  100", "", MB_OK);
            return TRUE;
          }
          else
          {
            numstars = n;
            for (i = 0; i < numstars; i++)
              InitStar(i);
            EndDialog(hWnd, IDOK);
          }
          break;
        case IDCANCEL: EndDialog(hWnd, IDCANCEL);
      }
  }

  return FALSE;
}

void ParamsDialog(HWND hWnd)
{
  DialogBox(MainInstance, (LPCTSTR)2, hWnd, (DLGPROC)dlgProc);
}

LONG WINAPI wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HDC hdc;
  static HGLRC hrc;
  PAINTSTRUCT ps;
  GLdouble aspect;
  GLsizei width, height;

  switch(msg)
  {
    case WM_CREATE:
		hdc = GetDC(hWnd);
		setDCPixelFormat(hdc);
		hrc = wglCreateContext(hdc);
   	wglMakeCurrent(hdc, hrc);
      InitializeGL();
      timer = SetTimer(hWnd, 1, 50, NULL);
      return 0;

    case WM_SIZE:
      width = (GLsizei)LOWORD(lParam);
      height = (GLsizei)HIWORD(lParam);
      if (!width || !height) return 0;

      aspect = (GLdouble)width /(GLdouble)height;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(30.0, aspect, 1.0, 100.0);
      glViewport(0, 0, width, height);

      return 0;

    case WM_PAINT:
      BeginPaint(hWnd, &ps);
      drawScene(hdc);
      EndPaint(hWnd, &ps);
      return 0;

    case WM_DESTROY:
      wglMakeCurrent(NULL, NULL);
      wglDeleteContext(hrc);
      KillTimer(hWnd, timer);
      PostQuitMessage(0);
      return 0;

    case WM_TIMER:
      InvalidateRect(hWnd, NULL, FALSE);
      return 0;

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case 101: ParamsDialog(hWnd); break;
        case 102: DestroyWindow(hWnd);
      }
      return 0;
  }

  return DefWindowProc(hWnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow)
{
  static char appName [] = "Stars - by HackMaster";
  WNDCLASS wc;
  HWND hWnd;
  MSG msg;

  randomize();
  MainInstance = hInstance;

  for (int i = 0; i < NUMSTARS; i++)
    InitStar(i);

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc =(WNDPROC)wndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground =(HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = (LPCTSTR)1;
  wc.lpszClassName = appName;

  RegisterClass(&wc);

  hWnd = CreateWindow(appName, appName,
                      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                      CW_USEDEFAULT, CW_USEDEFAULT, 320,
                      240, HWND_DESKTOP, NULL, hInstance, NULL);

  ShowWindow(hWnd, cmdShow);
  UpdateWindow(hWnd);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return msg.wParam;
}
